// WhaleTeqSEEG_SDK.h

#ifdef WHALETEQSEEG_SDK_EXPORTS
#define WHALETEQSEEG_SDK_API __declspec(dllexport) 
#else
#define WHALETEQSEEG_SDK_API __declspec(dllimport) 
#endif

extern "C"{
	typedef void (__stdcall *DeviceConnectedCB)(bool IsConnected);
    WHALETEQSEEG_SDK_API void RegistConnectCB(DeviceConnectedCB connCB);

	WHALETEQSEEG_SDK_API bool TriggerDeviceChanged();

	WHALETEQSEEG_SDK_API bool InitSEEG();
	WHALETEQSEEG_SDK_API bool IsEEG100E();
	WHALETEQSEEG_SDK_API void ResetSEEG();
	WHALETEQSEEG_SDK_API void CloseSEEG();

	WHALETEQSEEG_SDK_API char* GetUSBPort();

	WHALETEQSEEG_SDK_API char* GetSerialNo();

	WHALETEQSEEG_SDK_API char* GetHWInfo();
	WHALETEQSEEG_SDK_API char* GetFWVersion();
	WHALETEQSEEG_SDK_API char* GetHWVersion();

	//Parameter - time
	//	unit: second
	//	time from the output signal changed(function, amplitude, frequency ...)
	//Parameter - amp
	//	unit: millivolt(mV)
	typedef void (__stdcall *TimeAmplitudeCB)(double time, double amp);

	//About 0.006 second(6ms) interval between each callback be triggered
	WHALETEQSEEG_SDK_API void RegisterTimeAmpCB(TimeAmplitudeCB cb);

	enum OutputFunctionEEG_E {
		EEG_Output_Off	= 0,
		EEG_Output_Sine,
		EEG_Output_Triangle,
		EEG_Output_Square,
		EEG_Output_RectanglePulse,
		EEG_Output_TrianglePulse,
        EEG_Output_Hysteresis,
        EEG_Output_PlayWaveform
	};
	
	//Return value:
	//	0:	succeed
	//	-1:	SEEG not be initialized
	WHALETEQSEEG_SDK_API int SetOutputFunc(OutputFunctionEEG_E func);
	WHALETEQSEEG_SDK_API OutputFunctionEEG_E GetOutputFunc();

	//Parameter - amp:
	//	value range: -2000 ~ 2000
	//	unit: microvolt(uV)
	//Return value:
	//	0: succeed
	//	-1: SEEG not be initialized
	//	-2: value out of range
	//  -3: "Baseline Reset Test" is working
	//Default value after InitSEEG(): 100uV
	WHALETEQSEEG_SDK_API int SetAmplitude(double amp);
	WHALETEQSEEG_SDK_API double GetAmplitude();
	#define DefaultAmplitude	100.0
	#define MaxAmplitude		2000.0
	#define MinAmplitude		-2000.0

	//Parameter - freq:
	//	value range: 0.05 ~ 500
	//               0.05 ~ 3 when Output as Pulse Waveforms
	//	unit: hertz
	//Return value:
	//	0: succeed
	//	-1: SEEG not be initialized
	//	-2: value out of range
	//	-3: "Frequency Scan" is working
	//  -4: "Baseline Reset Test" is working
	//Default value after InitSEEG(): 10Hz
	WHALETEQSEEG_SDK_API int SetFrequency(double freq);
	WHALETEQSEEG_SDK_API double GetFrequency();
	#define DefaultFrequency		10.0
	#define	MaxFrequency			500.0
	#define MinFrequency			0.05
	#define	MaxFrequencyPulseWave	3.0

	//Parameter - setting:
	//	value range: -300 ~ 300 - DCOffset Variable mode Off
	//				 -150 ~ 150 - DCOffset Variable mode Off for 100E
	//               -1000 ~ 1000 - DCOffset Variable mode On
	//	unit: millivolt(mV)
	//Return value:
	//	0: succeed
	//	-1: SEEG not be initialized
	//	-2: value out of range
	//Default value after InitSEEG(): 0mV
	WHALETEQSEEG_SDK_API int SetDCOffset(int setting);
	WHALETEQSEEG_SDK_API int GetDCOffset();
	#define DefaultDCOffset		0
	#define MaxDCOffset			300
	#define MinDCOffset			-300
	#define MaxDCOffset_100E	150
	#define MinDCOffset_100E	-150
	#define MaxDCOffsetVarMode	1000
	#define MinDCOffsetVarMode	-1000

	//Return value:
	//	0: succeed
	//	-1: SEEG not be initialized
	//Default value after InitSEEG(): false(Off)
	WHALETEQSEEG_SDK_API int SetDCOffsetVariableMode(bool OnOff);
	WHALETEQSEEG_SDK_API bool GetDCOffsetVariableMode();

	enum OutputLead_E {
		Lead_Ch1		= 0,
		Lead_Ch2,
		Lead_Ch3,
		Lead_Ch4,
		Lead_Ch5,
		Lead_Ch6,
		Lead_Ch7,
		Lead_Ch8,
		Lead_Ch9  
        //Channel 10~22 are shorted together
	};
	//Return value:
	//	0: succeed
	//	-1: SEEG not be initialized
	//Default value after InitSEEG(): Lead_Ch1 - On; all others - Off;
	WHALETEQSEEG_SDK_API int SetOutputLead(OutputLead_E lead, bool OnOff);
	WHALETEQSEEG_SDK_API bool GetOutputLead(OutputLead_E lead);

	//Parameter - width:
	//	value range: 2 ~ 300
	//	unit: milisecond(ms)
	//Return value:
	//	0: succeed
	//	-1: SEEG not be initialized
	//	-2: value out of range
	//Default value after InitSEEG(): 100ms
	WHALETEQSEEG_SDK_API int SetPulseWidth(double width);
	WHALETEQSEEG_SDK_API double GetPulseWidth();
	#define DefaultPulseWidth	100
	#define MaxPulseWidth		300
	#define MinPulseWidth		2

	//Return value:
	//	0: succeed
	//	-1: SEEG not be initialized
	//Default value after InitSEEG(): true(On - 620k&/4.7nF shorted)
	WHALETEQSEEG_SDK_API int SetInputImpedanceMode(bool OnOff);
	WHALETEQSEEG_SDK_API bool GetInputImpedanceMode();

	//Parameter - freq
	//	unit: hertz
	//	frequency value changed during "Baseline Reset Test" or "Frequency Scanning"
	typedef void (__stdcall *SyncFreqCB)(double freq);

	WHALETEQSEEG_SDK_API void RegisterSyncFreqCB(SyncFreqCB cb);

	enum BaselineReset_E {
		Baseline_Off	= 0,
		Baseline_50Hz,
		Baseline_60Hz,
		Baseline_80Hz,
		Baseline_100Hz
	};
	//Return value:
	//	0: succeed
	//	-1: SEEG not be initialized
	//	-2: current main output function not be "Output_Sine"
	//Default value after InitSEEG(): Baseline_Off
	//"Output Function" must be set to "Output_Sine" before setting as "non - Baseline_Off" value
	//After setting as "non - Baseline_Off" value, "Amplitude" would be locked to "2.5mV"
	//and "Frequency" would be locked as "50/60/80/100" Hz.
	WHALETEQSEEG_SDK_API int SetBaselineResetTest(BaselineReset_E mode);
	WHALETEQSEEG_SDK_API BaselineReset_E GetBaselineResetTest();

	//Parameter - func
	//	main function changed during "Frequency Scanning"
	typedef void (__stdcall *SyncMainFuncCB)(int func);

	WHALETEQSEEG_SDK_API void RegisterSyncMainFuncCB(SyncMainFuncCB cb);

	//Parameter - startFreq:
	//	value range: 0.5 ~ 500
	//	unit: hertz
	//	Default value after InitSEEG(): 0.5Hz
	//Parameter - stopFreq:
	//	value range: 0.05 ~ 500
	//	unit: hertz
	//	Default value after InitSEEG(): 150Hz
	//Parameter - duration:
	//	value range: 10 ~ 180
	//	unit: milisecond(ms)
	//	Default value after InitSEEG(): 30ms
	//Return value:
	//	0: succeed
	//	-1: SEEG not be initialized
	//	-2: startFreq out of range
	//	-3: stopFreq out of range
	//	-4: duration out of range
	//"Output Function" would be set to "Output_Sine" when setting "OnOff" as true,
	//then "Frequency" would be locked(SetFrequency() return -3) for periodly scanning change.
	//"Output Function" would be set to "Output_Off" when setting "OnOff" as false.
	WHALETEQSEEG_SDK_API int SetFreqScanSine(bool OnOff, double startFreq, 
											 double stopFreq, int duration);
	WHALETEQSEEG_SDK_API bool GetFreqScanSine();
	WHALETEQSEEG_SDK_API double GetFreqScanSineStartFreq();
	WHALETEQSEEG_SDK_API double GetFreqScanSineStopFreq();
	WHALETEQSEEG_SDK_API int GetFreqScanSineDuration();
	#define DefaultScanSineStartFreq	0.5
	#define DefaultScanSineStopFreq		150
	#define MaxScanSineFreq				500.0
	#define MinScanSineFreq				0.5
	#define DefaultScanSineDuration		30
	#define MaxScanSineDuration			180
	#define MinScanSineDuration			10

	//Return value:
	// >0: succeed, the number of sample data loaded
	// -1: Open file failed
	// -2: Can't solve sample rate in Line 1
	// -3: Can't solve sample number in Line 2
    // -4: Raw data file is too large to fit the memory
	WHALETEQSEEG_SDK_API int LoadEEGtxt(wchar_t* file);
	WHALETEQSEEG_SDK_API int LoadEEGbin(wchar_t* file);

	//Return value:
	// >0: succeed, the number of sample data loaded
	// -1: Open file failed
	// -2: Can't solve signal corresponding to signalIdx
    // -4: Raw data file is too large to fit the memory
	WHALETEQSEEG_SDK_API int LoadEEGedf(wchar_t* file, int signalIdx);

	WHALETEQSEEG_SDK_API char* GetSDKVersion();
}







