﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;    // For StructLayout, DllImport
using System.IO;
using ZedGraph;
using WhaleTeqSEEG_SDK;

namespace WhaleteqSEEG_SDK_UI_Demo
{
    public partial class EEG_SDK_DemoForm : Form
    {
        public delegate void ConnectCBDelegte(bool IsConnect);
        private static ConnectCBDelegte fpConnect; //avoid CB be GC

        public delegate void TimeAmplitudeCBDelegate(double time, double amp);
        private static TimeAmplitudeCBDelegate fpTimeAmp; //avoid CB be GC

        public delegate void SyncFreqCBDelegate(double freq);
        private static SyncFreqCBDelegate fpSyncFreq; //avoid CB be GC

        public delegate void SyncMainFuncCBDelegate(OutputFunction mainFunc);
        private static SyncMainFuncCBDelegate fpSyncMainFunc; //avoid CB be GC

        private bool isSync = false;
        private bool ignoreSyncOff = false;

        public ZedGraphControl Z;

        private void ConnectCB(bool IsConnect)
        {
            if (!IsConnect)
            {
                this.Text = "SEEG SDK Demo           Can't initialize SEEG!";
            }
            else
            {
                this.Text = "SEEG SDK Demo           SN: ";

                IntPtr ptr;
                if (IntPtr.Zero != (ptr = SEEG_SDK.GetHWInfo()))
                    this.Text += Marshal.PtrToStringAnsi(ptr);
                else
                    this.Text += SEEG_SDK.GetSerialNo();

                this.Text += "   SDK Version: " + SEEG_SDK.GetSDKVersion();

                if (SEEG_SDK.IsEEG100E())
                {
                    this.Text += "  - 100E";

                    DCSetting.Maximum = 150;
                    DCSetting.Minimum = -150;
                    DCSetting.Increment = 150;
                }
                else
                {
                    DCSetting.Maximum = 300;
                    DCSetting.Minimum = -300;
                    DCSetting.Increment = 300;
                }
            }
        }

        private void _connectCB(bool IsConnect)
        {
            if (InvokeRequired)
            {
                ConnectCBDelegte cb = new ConnectCBDelegte(ConnectCB);
                try
                {
                    Invoke(cb, IsConnect);
                }
                catch { };
            }
            else
            {
                ConnectCB(IsConnect);
            }
        }

        public EEG_SDK_DemoForm()
        {
            InitializeComponent();
            AddGraphs();

            //_initSEEG();
            ConnectCB(SEEG_SDK.InitSEEG());

            fpConnect = new ConnectCBDelegte(_connectCB);
            IntPtr ipConnect = Marshal.GetFunctionPointerForDelegate(fpConnect);
            SEEG_SDK.RegistConnectCB(ipConnect);

            fpTimeAmp = new TimeAmplitudeCBDelegate(TimeAmplitudeCB);
            IntPtr ipTimeAmp = Marshal.GetFunctionPointerForDelegate(fpTimeAmp);
            SEEG_SDK.RegisterTimeAmpCB(ipTimeAmp);

            fpSyncFreq = new SyncFreqCBDelegate(SyncFreqCB);
            IntPtr ipSyncFreq = Marshal.GetFunctionPointerForDelegate(fpSyncFreq);
            SEEG_SDK.RegisterSyncFreqCB(ipSyncFreq);

            fpSyncMainFunc = new SyncMainFuncCBDelegate(SyncMainFuncCB);
            IntPtr ipSyncMainFunc = Marshal.GetFunctionPointerForDelegate(fpSyncMainFunc);
            SEEG_SDK.RegisterSyncMainFuncCB(ipSyncMainFunc);
        }

        void AddGraphs()
        {
            Z = new ZedGraphControl();
            this.Controls.Add(Z);
            Z.Location = new Point(12, 445);
            Z.Size = new Size(560, 180);
            Z.ScrollGrace = 0;
            Z.ScrollMaxX = 0;
            Z.ScrollMaxY = 0;
            Z.ScrollMaxY2 = 0;
            Z.ScrollMinX = 0;
            Z.ScrollMinY = 0;
            Z.ScrollMinY2 = 0;
            GraphPane GP = Z.GraphPane;
            LineItem M = GP.AddCurve("", new PointPairList(), Color.Blue, SymbolType.None);
            LineItem N = GP.AddCurve("", new PointPairList(), Color.DarkViolet, SymbolType.None);
            GP.Title.Text = "Output";
            GP.YAxis.Title.Text = "mV";
            GP.XAxis.Title.Text = "Time (s)";
            double XS = 8;
            Axis X = Z.GraphPane.XAxis;
            X.Scale.Max = XS;
            X.Scale.MajorStep = 1;
            X.Scale.MinorStep = 0.2;
            X.MinorGrid.IsVisible = true;
            X.MinorGrid.DashOff = 1;
            X.MinorGrid.DashOn = 8;
            X.MinorGrid.Color = Color.Pink;
            X.MajorGrid.IsVisible = true;
            X.MajorGrid.DashOff = 0;
            X.MajorGrid.DashOn = 16;
            X.MajorGrid.Color = Color.Pink;
            UpdateGraphScale(this, null);
        }

        void ResetGraph()
        {
            //this.grp_special_waveform_monitor.Visible = (Function == "ECGFile");
            //tg = 0;
            Z.GraphPane.CurveList[0].Clear();
            Z.GraphPane.CurveList[1].Clear();
        }

        private double YS;
        public void UpdateGraphScale(object O, EventArgs e)
        {
            Axis Y = Z.GraphPane.YAxis;
            YS = 0.4;
            if (Graph20.Checked) YS = 0.2;
            if (Graph4.Checked) YS = 1.0;
            Y.Scale.Max = 2 * YS;
            Y.Scale.Min = 0;
            Y.Scale.MajorStep = 0.1;
            Y.Scale.MinorStep = 0.05;
            if (Graph20.Checked)
            {
                Y.Scale.MajorStep = 0.1;
                Y.Scale.MinorStep = 0.05;
            }
            if (Graph10.Checked)
            {
                Y.Scale.MajorStep = 0.2;
                Y.Scale.MinorStep = 0.1;
            }
            if (Graph4.Checked)
            {
                Y.Scale.MajorStep = 0.2;
                Y.Scale.MinorStep = 0.1;
            }
            Y.MinorGrid.IsVisible = true;
            Y.MajorGrid.IsVisible = true;
            Y.MinorGrid.Color = Color.Pink;
            Y.MajorGrid.Color = Color.Pink;
            Y.IsAxisSegmentVisible = false;
            Y.MinorGrid.DashOff = 1;
            Y.MinorGrid.DashOn = 5;
            Y.MajorGrid.DashOff = 0;
            Y.MajorGrid.DashOn = 10;
            ResetGraph();
            Z.Invalidate();
        }

        private const int ZBufSize = 64;
        private double[] ZtgBuf = new double[ZBufSize];
        private double[] ZampBuf = new double[ZBufSize];
        private int ZBufIdx = 0;
        private double lastTg = 0;

        private void TimeAmplitudeCB(double time, double amp)
        {
            double tg = time - ((int)time / 8) * 8;

            if (ZBufIdx == ZBufSize || tg < lastTg)
            {
                if (Z.GraphPane.CurveList[0].NPts > 0 &&
                    ZtgBuf[0] < Z.GraphPane.CurveList[0].Points[Z.GraphPane.CurveList[0].NPts - 1].X)
                {
                    Z.GraphPane.CurveList[0].Clear();
                    Z.GraphPane.CurveList[1].Clear();
                }

                for (int i = 0; i < ZBufIdx; i++)
                    Z.GraphPane.CurveList[0].AddPoint(ZtgBuf[i], ZampBuf[i] + YS);

                ZBufIdx = 0;

                sNeedUpdateZ = true;
                sTime = time;
            }

            ZtgBuf[ZBufIdx] = tg;
            ZampBuf[ZBufIdx++] = amp;

            lastTg = tg;
        }

        private bool sNeedUpdateZ = false;
        private double sTime = 0;

        private void timer_updateGUI_Tick(object sender, EventArgs e)
        {
            if (sNeedUpdateZ)
            {
                Z.Invalidate();

                String timeStr = sTime.ToString("000.0") + "s";
                if (timeStr != timeTextBox.Text)
                    timeTextBox.Text = timeStr;

                sNeedUpdateZ = false;
            }
        }

        public delegate void SyncFreqDelegate(double freq);
        private void SyncFreqCB(double freq)
        {
            if (FreqHz.InvokeRequired)
            {
                SyncFreqDelegate cb = new SyncFreqDelegate(SyncFreqCB);
                try
                {
                    FreqHz.Invoke(cb, freq);
                }
                catch { };
            }
            else
            {
                isSync = true;

                FreqHz.Value = (decimal)freq;

                isSync = false;
            }
        }

        public delegate void SyncMainFuncDelegate(OutputFunction mainFunc);
        private void SyncMainFuncCB(OutputFunction mainFunc)
        {
            if (OutputOff.InvokeRequired)
            {
                SyncMainFuncDelegate cb = new SyncMainFuncDelegate(SyncMainFuncCB);
                try
                {
                    OutputOff.Invoke(cb, mainFunc);
                }
                catch { };
                return;
            }

            if (mainFunc == OutputFunction.Output_Off)
            {
                if (ignoreSyncOff)
                    ignoreSyncOff = false;
                else
                    OutputOff.Checked = true;
                return;
            }

            isSync = true;

            switch (mainFunc)
            {
                case OutputFunction.Output_Sine:
                    OutputSine.Checked = true;
                    break;
                case OutputFunction.Output_Triangle:
                    OutputTriangle.Checked = true;
                    break;
                case OutputFunction.Output_Square:
                    OutputSquare.Checked = true;
                    break;
                case OutputFunction.Output_RectanglePulse:
                    OutputRectanglePulse.Checked = true;
                    break;
                case OutputFunction.Output_TrianglePulse:
                    OutputTrianglePulse.Checked = true;
                    break;
                case OutputFunction.Output_Hysteresis:
                    OutputHysteresis.Checked = true;
                    break;
                case OutputFunction.Outpue_PlayWaveform:
                    button_PlayWaveform.Enabled = true;
                    break;
            }

            isSync = false;
        }

        private void EEG_SDK_DemoForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            SEEG_SDK.CloseSEEG();
        }

        private void OutputOff_CheckedChanged(object sender, EventArgs e)
        {

            RadioButton btn = (RadioButton)sender;

            if (!btn.Checked) return;

            if (isSync) return;

            SEEG_SDK.SetOutputFunc(OutputFunction.Output_Off);

            FreqHz.Maximum = 500;

            if (Baseline1V.Checked)
                Baseline1V.Checked = false;
        }

        private void OutputSine_CheckedChanged(object sender, EventArgs e)
        {
            RadioButton btn = (RadioButton)sender;

            Baseline1V.Enabled = btn.Checked;

            if (!btn.Checked)
            {
                if (FreqScanSine.Checked)
                {
                    ignoreSyncOff = true;
                    FreqScanSine.Checked = false;
                }

                return;
            }

            if (isSync) return;

            SEEG_SDK.SetOutputFunc(OutputFunction.Output_Sine);

            FreqHz.Maximum = 500;
        }

        private void OutputTirangle_CheckedChanged(object sender, EventArgs e)
        {
            RadioButton btn = (RadioButton)sender;
            if (!btn.Checked) return;

            if (isSync) return;

            SEEG_SDK.SetOutputFunc(OutputFunction.Output_Triangle);

            FreqHz.Maximum = 500;
        }

        private void OutputSquare_CheckedChanged(object sender, EventArgs e)
        {
            RadioButton btn = (RadioButton)sender;
            if (!btn.Checked) return;

            if (isSync) return;

            SEEG_SDK.SetOutputFunc(OutputFunction.Output_Square);

            FreqHz.Maximum = 500;
        }

        private void OutputRectanglePulse_CheckedChanged(object sender, EventArgs e)
        {
            RadioButton btn = (RadioButton)sender;
            if (!btn.Checked) return;

            if (isSync) return;

            SEEG_SDK.SetOutputFunc(OutputFunction.Output_RectanglePulse);

            FreqHz.Maximum = 3;
        }

        private void OutputTrianglePulse_CheckedChanged(object sender, EventArgs e)
        {
            RadioButton btn = (RadioButton)sender;
            if (!btn.Checked) return;

            if (isSync) return;

            SEEG_SDK.SetOutputFunc(OutputFunction.Output_TrianglePulse);

            FreqHz.Maximum = 3;
        }

        private void OutputHysteresis_CheckedChanged(object sender, EventArgs e)
        {
            RadioButton btn = (RadioButton)sender;
            if (!btn.Checked) return;

            if (isSync) return;

            SEEG_SDK.SetOutputFunc(OutputFunction.Output_Hysteresis);

            FreqHz.Maximum = 3;
        }

        private void Amplitude_mu_ValueChanged(object sender, EventArgs e)
        {
            NumericUpDown num = (NumericUpDown)sender;
            SEEG_SDK.SetAmplitude((double)num.Value);

            double a_mu = (double)Amplitude_mu.Value / 1000D;
            decimal temp_a = (decimal)a_mu;
            Amplitude.Value = temp_a;
        }

        private void Amp_ValueChanged(object sender, EventArgs e)
        {
            Amplitude_mu.Value = Amplitude.Value * 1000; 
        }

        private void FreqHz_ValueChanged(object sender, EventArgs e)
        {
            if (!isSync)
            {
                SEEG_SDK.SetFrequency((double)FreqHz.Value);
            }
        }

        private void PulseWidth_ValueChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetPulseWidth((double)PulseWidth.Value);
        }

        private void DCSetting_ValueChanged(object sender, EventArgs e)
        {
            NumericUpDown num = (NumericUpDown)sender;
            SEEG_SDK.SetDCOffset((int)num.Value);
        }

        private void DCVariable_CheckedChanged(object sender, EventArgs e)
        {
            if (DCVariable.Checked)
            {
                DCSetting.Maximum = 1000;
                DCSetting.Minimum = -1000;
                DCSetting.Increment = 5;
            }
            else
            {
                DCSetting.Value = 0;
                if (SEEG_SDK.IsEEG100E())
                {
                    DCSetting.Maximum = 150;
                    DCSetting.Minimum = -150;
                    DCSetting.Increment = 150;
                }
                else
                {
                    DCSetting.Maximum = 300;
                    DCSetting.Minimum = -300;
                    DCSetting.Increment = 300;
                }
            }

            SEEG_SDK.SetDCOffsetVariableMode(DCVariable.Checked);
        }

        private void InputImpedance_CheckedChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetInputImpedanceMode(InputImpedance.Checked);
        }

        private void LeadCh1_CheckedChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetOutputLead(OutputLead_E.Lead_Ch1, LeadCh1.Checked);
        }

        private void LeadCh2_CheckedChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetOutputLead(OutputLead_E.Lead_Ch2, LeadCh2.Checked);
        }

        private void LeadCh3_CheckedChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetOutputLead(OutputLead_E.Lead_Ch3, LeadCh3.Checked);
        }

        private void LeadCh4_CheckedChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetOutputLead(OutputLead_E.Lead_Ch4, LeadCh4.Checked);
        }

        private void LeadCh5_CheckedChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetOutputLead(OutputLead_E.Lead_Ch5, LeadCh5.Checked);
        }

        private void LeadCh6_CheckedChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetOutputLead(OutputLead_E.Lead_Ch6, LeadCh6.Checked);
        }

        private void LeadCh7_CheckedChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetOutputLead(OutputLead_E.Lead_Ch7, LeadCh7.Checked);
        }

        private void LeadCh8_CheckedChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetOutputLead(OutputLead_E.Lead_Ch8, LeadCh8.Checked);
        }

        private void LeadCh9_CheckedChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetOutputLead(OutputLead_E.Lead_Ch9, LeadCh9.Checked);
        }

        void Baseline1V_CheckedChanged(object sender, EventArgs e)
        {
            BaselineReset_E eOption = BaselineReset_E.Baseline_Off;
            if (Baseline1V.Checked)
            {
                if (N50Hz.Checked)
                    eOption = BaselineReset_E.Baseline_50Hz;
                else if (N60Hz.Checked)
                    eOption = BaselineReset_E.Baseline_60Hz;
                else if (N80Hz.Checked)
                    eOption = BaselineReset_E.Baseline_80Hz;
                else if (N100Hz.Checked)
                    eOption = BaselineReset_E.Baseline_100Hz;
            }

            SEEG_SDK.SetBaselineResetTest(eOption);
        }

        void FreqScanSine_CheckedChanged(object sender, EventArgs e)
        {
            SEEG_SDK.SetFreqScanSine(FreqScanSine.Checked, (double)ScanStartFreq.Value, (double)ScanStopFreq.Value, (int)ScanDuration.Value);
        }

        private void btn_load_waveform_Click(object sender, EventArgs e)
        {
            OpenFileDialog FD = new OpenFileDialog();

            FD.Filter = "Text format|*.txt|Binary format|*.bin";
            FD.ShowDialog();
            if (FD.FileName == "") return;
            if (new FileInfo(FD.FileName).Extension == ".txt")
            {
                if (SEEG_SDK.LoadEEGtxt(FD.FileName) >= 0)
                    button_PlayWaveform.Enabled = true;
            }
            else if (new FileInfo(FD.FileName).Extension == ".bin")
            {
                if (SEEG_SDK.LoadEEGbin(FD.FileName) >= 0)
                    button_PlayWaveform.Enabled = true;
            }
        }

        private void button_PlayWaveform_Click(object sender, EventArgs e)
        {
            OutputOff.Checked = false;
            SEEG_SDK.SetOutputFunc(OutputFunction.Outpue_PlayWaveform);
        }

        private void ResetDefaultValue_Click(object sender, EventArgs e)
        {
            if (SEEG_SDK.GetOutputFunc() != OutputFunction.Output_Off)
                OutputOff.Checked = true;

            if (SEEG_SDK.GetAmplitude() != SEEG_SDK.DefaultAmplitude)
                Amplitude_mu.Value = (decimal)SEEG_SDK.DefaultAmplitude;

            if (SEEG_SDK.GetFrequency() != SEEG_SDK.DefaultFrequency)
                FreqHz.Value = (decimal)SEEG_SDK.DefaultFrequency;

            if (SEEG_SDK.GetDCOffset() != SEEG_SDK.DefaultDCOffset)
                DCSetting.Value = (decimal)SEEG_SDK.DefaultDCOffset;

            if (SEEG_SDK.GetDCOffsetVariableMode())
                DCVariable.Checked = false;

            if (!SEEG_SDK.GetOutputLead(OutputLead_E.Lead_Ch1))
                LeadCh1.Checked = true;
            if (SEEG_SDK.GetOutputLead(OutputLead_E.Lead_Ch2))
                LeadCh2.Checked = false;
            if (SEEG_SDK.GetOutputLead(OutputLead_E.Lead_Ch3))
                LeadCh3.Checked = false;
            if (SEEG_SDK.GetOutputLead(OutputLead_E.Lead_Ch4))
                LeadCh4.Checked = false;
            if (SEEG_SDK.GetOutputLead(OutputLead_E.Lead_Ch5))
                LeadCh5.Checked = false;
            if (SEEG_SDK.GetOutputLead(OutputLead_E.Lead_Ch6))
                LeadCh6.Checked = false;
            if (SEEG_SDK.GetOutputLead(OutputLead_E.Lead_Ch7))
                LeadCh7.Checked = false;
            if (SEEG_SDK.GetOutputLead(OutputLead_E.Lead_Ch8))
                LeadCh8.Checked = false;
            if (SEEG_SDK.GetOutputLead(OutputLead_E.Lead_Ch9))
                LeadCh9.Checked = false;

            if (SEEG_SDK.GetFreqScanSineStartFreq() != SEEG_SDK.DefaultScanSineStartFreq)
                ScanStartFreq.Value = (decimal)SEEG_SDK.DefaultScanSineStartFreq;
            if (SEEG_SDK.GetFreqScanSineStopFreq() != SEEG_SDK.DefaultScanSineStopFreq)
                ScanStopFreq.Value = (decimal)SEEG_SDK.DefaultScanSineStopFreq;
            if (SEEG_SDK.GetFreqScanSineDuration() != SEEG_SDK.DefaultScanSineDuration)
                ScanDuration.Value = (decimal)SEEG_SDK.DefaultScanSineDuration;

            ResetGraph();
            Z.Invalidate();

            timeTextBox.Text = "000.0s";

            SEEG_SDK.ResetSEEG();
        }


    }
}
