﻿using System;
using System.Runtime.InteropServices;    // For StructLayout, DllImport

namespace WhaleTeqSEEG_SDK
{
    public enum OutputFunction
    {
        Output_Off  = 0,
        Output_Sine,
        Output_Triangle,
        Output_Square,
        Output_RectanglePulse,
        Output_TrianglePulse,
        Output_Hysteresis,
        Outpue_PlayWaveform
    };

    enum OutputLead_E 
    {
        Lead_Ch1 = 0,
        Lead_Ch2,
        Lead_Ch3,
        Lead_Ch4,
        Lead_Ch5,
        Lead_Ch6,
        Lead_Ch7,
        Lead_Ch8,
        Lead_Ch9
        //Channel 10~22 are shorted together
	};

    enum BaselineReset_E 
    {
		Baseline_Off	= 0,
		Baseline_50Hz,
		Baseline_60Hz,
		Baseline_80Hz,
		Baseline_100Hz
	};   

    class SEEG_SDK
    {
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.LPStr)]
        public static extern string GetSDKVersion();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void RegistConnectCB(IntPtr cb);
        //Parameter - cb:
        //  function pointer of delegate from "Marshal.GetFunctionPointerForDelegate(delegate)"
        //      delegate void ConnectCBDelegate(bool IsConnected);
        //      Parameter - IsConnected
        //	        true for connected;
        //	        false for disconnected;


        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool InitSEEG();
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool IsEEG100E();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void ResetSEEG();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void CloseSEEG();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr GetUSBPort();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.LPStr)]
        public static extern string GetSerialNo();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr GetHWInfo();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr GetFWVersion();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr GetHWVersion();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void RegisterTimeAmpCB(IntPtr cb);
        //Parameter - cb:
        //  function pointer of delegate from "Marshal.GetFunctionPointerForDelegate(delegate)"
        //      delegate void TimeAmplitudeCBDelegate(double time, double amp);
        //      Parameter - time
        //	        unit: second
        //	        time from the output signal changed(function, amplitude, frequency ...)
        //      Parameter - amp
        //	        unit: millivolt(mV)
        //About 0.006 second(6ms) interval between each callback be triggered

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int SetOutputFunc(OutputFunction func);
        //Return value:
        //	0:	succeed
        //	-1:	SEEG not be initialized
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern OutputFunction GetOutputFunc();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int SetAmplitude(double amp);
        //Parameter - amp:
        //	value range: -2000 ~ 2000
        //	unit: microvolt(uV)
        //Return value:
        //	0: succeed
        //	-1: SEEG not be initialized
        //	-2: value out of range
        //  -3: "Baseline Reset Test" is working
        //Default value after InitSEEG(): 100 uV
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern double GetAmplitude();
        public const double DefaultAmplitude = 100;

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int SetFrequency(double freq);
        //Parameter - freq:
        //	value range: 0.05 ~ 500
        //               0.05 ~ 3 when Output as Pulse Waveforms
        //	unit: hertz
        //Return value:
        //	0: succeed
        //	-1: SEEG not be initialized
        //	-2: value out of range
        //	-3: "Frequency Scan" is working
        //  -4: "Baseline Reset Test" is working
        //Default value after InitSEEG(): 10 Hz
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern double GetFrequency();
        public const double DefaultFrequency = 10.0;

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int SetDCOffset(int setting);
        //Parameter - setting:
        //	value range: -300 ~ 300 - DCOffset Variable mode Off
        //				 -150 ~ 150 - DCOffset Variable mode Off for 100E
        //               -1000 ~ 1000 - DCOffset Variable mode On
        //	unit: millivolt(mV)
        //Return value:
        //	0: succeed
        //	-1: SEEG not be initialized
        //	-2: value out of range
        //Default value after InitSEEG(): 0mV
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int GetDCOffset();
        public const int DefaultDCOffset = 0;

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int SetDCOffsetVariableMode(bool OnOff);
        //Return value:
        //	0: succeed
        //	-1: SEEG not be initialized
        //Default value after InitSEEG(): false(Off)
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool GetDCOffsetVariableMode();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int SetPulseWidth(double width);
        //Parameter - width:
        //	value range: 2 ~ 300
        //	unit: milisecond(ms)
        //Return value:
        //	0: succeed
        //	-1: SEEG not be initialized
        //	-2: value out of range
        //Default value after InitSEEG(): 100ms
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern double GetPulseWidth();
        public const double DefaultPulseWidth = 100;

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int SetInputImpedanceMode(bool OnOff);
        //Return value:
        //	0: succeed
        //	-1: SEEG not be initialized
        //Default value after InitSEEG(): true(On - 620k&/4.7nF shorted)
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool GetInputImpedanceMode();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
	    public static extern int SetOutputLead(OutputLead_E lead, bool OnOff);
	    //Return value:
	    //	0: succeed
	    //	-1: SEEG not be initialized
	    //Default value after InitSEEG(): Lead_Ch1 - On; all others - Off;
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool GetOutputLead(OutputLead_E lead);

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void RegisterSyncFreqCB(IntPtr cb);
        //Parameter - cb:
        //  function pointer of delegate from "Marshal.GetFunctionPointerForDelegate(delegate)"
        //      delegate void SyncFreqCBDelegate(double freq);
        //      Parameter - freq
        //	        unit: hertz
        //	        frequency value changed during "Baseline Reset Test" or "Frequency Scanning"


        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
	    public static extern int SetBaselineResetTest(BaselineReset_E mode);        	
	    //Return value:
	    //	0: succeed
	    //	-1: SEEG not be initialized
	    //	-2: current main output function not be "Output_Sine"
	    //Default value after InitSEEG(): Baseline_Off
	    //"Output Function" must be set to "Output_Sine" before setting as "non - Baseline_Off" value
	    //After setting as "non - Baseline_Off" value, "Amplitude" would be locked to "2.5mV"
	    //and "Frequency" would be locked as "50/60/80/100" Hz.
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern BaselineReset_E GetBaselineResetTest();

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void RegisterSyncMainFuncCB(IntPtr cb);
        //Parameter - cb:
        //  function pointer of delegate from "Marshal.GetFunctionPointerForDelegate(delegate)"
        //      delegate void syncMainFuncCBDelegate(OutputFunction func);
        //      Parameter - func
        //	        main function changed during "Frequency Scanning"

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
	    public static extern int SetFreqScanSine(bool OnOff, double startFreq, 
											     double stopFreq, int duration);
        //Parameter - startFreq:
	    //	value range: 0.5 ~ 500
	    //	unit: hertz
	    //	Default value after InitSEEG(): 0.5Hz
	    //Parameter - stopFreq:
	    //	value range: 0.05 ~ 500
	    //	unit: hertz
	    //	Default value after InitSEEG(): 150Hz
	    //Parameter - duration:
	    //	value range: 10 ~ 180
	    //	unit: milisecond(ms)
	    //	Default value after InitSEEG(): 30ms
	    //Return value:
	    //	0: succeed
	    //	-1: SEEG not be initialized
	    //	-2: startFreq out of range
	    //	-3: stopFreq out of range
	    //	-4: duration out of range
	    //"Output Function" would be set to "Output_Sine" when setting "OnOff" as true,
	    //then "Frequency" would be locked(SetFrequency() return -3) for periodly scanning change.
	    //"Output Function" would be set to "Output_Off" when setting "OnOff" as false.
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool GetFreqScanSine();
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern double GetFreqScanSineStartFreq();
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern double GetFreqScanSineStopFreq();
        [DllImport(@"WhaleTeqSEEG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int GetFreqScanSineDuration();
        public const double DefaultScanSineStartFreq = 0.5;
        public const double DefaultScanSineStopFreq = 150;
        public const int DefaultScanSineDuration = 30;

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
	    public static extern int LoadEEGtxt(string file);
        // >0: succeed, the number of sample data loaded
        // -1: Open file failed
        // -2: Can't solve sample rate in Line 1
        // -3: Can't solve sample number in Line 2
        // -4: Raw data file is too large to fit the memory

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
	    public static extern int LoadEEGbin(string file);
        // >0: succeed, the number of sample data loaded
        // -1: Open file failed
        // -2: Can't solve sample rate in Line 1
        // -3: Can't solve sample number in Line 2
        // -4: Raw data file is too large to fit the memory

        [DllImport(@"WhaleTeqSEEG_SDK.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern int LoadEEGedf(string file, int signalIdx);
        // >0: succeed, the number of sample data loaded
        // -1: Open file failed
        // -2: Can't solve signal corresponding to signalIdx
        // -4: Raw data file is too large to fit the memory
    }
}
